unit ResScale;
{ Saves/Restores control positions & dimensions at design/run time for
  different screen resolutions, as well as form windowstate.
  Freeware Use & Abuse
  Author: Edward de la Rey
          edwardr@mailbox.ru.ac.za
          USE AT OWN RISK
  For Delphi 1,3 Will work with Delphi 2 if the Delphi 3 Specific controls
  are replaced with Delphi 2 Controls
  See RESSCALE.txt for usage}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms,Dialogs,
  stdctrls,DsgnIntf,CDialog,graphics,Registry;

  Const
  DefIniFileName = 'MyApp';
  Norm =0;Min = -1;Max =1; {WindowStates}
  Yes=1;No=0; {RestoreFormState} 

Type TControlsProperty = Class(TPropertyEditor)
Public
    function GetAttributes: TPropertyAttributes; override;
    Function GetValue:String;Override;
    procedure Edit;Override;
end;

type
  TResScale = class(TComponent)
  private
    { Private declarations }
    {TReaderProc = procedure(Reader: TReader) of object;
    TWriterProc = procedure(Writer: TWriter) of object; }
    GetScreenWidth:Word;
    GetScreenHeight:Word;
    GetIniFileName:String;
    FStorePositions:Boolean;
    FRestorePositions:Boolean;
    FCustomList:TControlsProperty;
    procedure WriteControlListData(Writer:TWriter);
    procedure FControlListData(Reader: TReader);
    procedure SetCustomList(Value:TControlsProperty);
    Procedure SetStorePositions (Value:Boolean);
    Procedure SetRestorePositions (Value:Boolean);
    Procedure SetScreenWidth (Value:Word);
    Procedure SetScreenHeight (Value:Word);
    Procedure SetIniFileName(Value:String);
  protected
    { Protected declarations }
    Procedure SaveComponentsPosition;
    Procedure LoadComponentsPosition;
    Function Decompose (var AString:String;Var it:String;Delimiter:String):Boolean;
  public
    procedure DefineProperties(Filer: TFiler);Override;
  published
    { Published declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    Property RegistryEntry:String Read GetIniFileName Write SetIniFileName;
    Property CurrentScreenWidth:Word Read GetScreenWidth Write SetScreenWidth;
    Property CurrentScreenHeight:Word Read GetScreenHeight Write SetScreenHeight;
    Property SavePositions:Boolean Read FStorePositions Write SetStorePositions;
    Property LoadPositions:Boolean Read FRestorePositions Write SetRestorePositions;
    Property ControlList:TControlsProperty Read FCustomList Write SetCustomList;
  end;

procedure Register;

var
MyControlList:TStringList;
TheOwner:tComponent;
RestoreFormState:Integer; {Yes,No}
FormState:Integer; {min,Norm,max}
DefLeft,DefHeight,DefTop,DefWidth:Integer;{Values for form in non max state}

implementation

function TControlsProperty.GetAttributes:TPropertyAttributes;
begin
  Result := [paDialog,PARevertable,paReadOnly];
end;

Function TControlsProperty.GetValue:String;
begin
 Result := ('[TControlList]');
end;

procedure TControlsProperty.Edit;
var
  ControlDialog: TControlDialog;
begin

  ControlDialog := TControlDialog.Create(Application);
  try
    ControlDialog.ShowModal; 
  finally
    ControlDialog.Free;
  end;

end;

procedure TResScale.WriteControlListData(Writer: TWriter);
var i:integer;
begin
  Writer.WriteListBegin;
  Writer.WriteInteger(RestoreFormState);  {MUST WE RESTORE FORM STATE}
  if MyControlList.Count > 0 then begin
       for I := 0 to myControlList.Count - 1 do
      Writer.WriteString(MyControlList[i]);
  end;
  Writer.WriteListEnd;
end;

procedure TResScale.FControlListData(Reader: TReader);
begin
  Reader.ReadListBegin;
  RestoreFormState:=Reader.ReadInteger; {Formstate variable}
  MyControlList.Clear;
  while not Reader.EndOfList do
    MyControlList.Add(Reader.ReadString);
  Reader.ReadListEnd;
end;

procedure TResScale.DefineProperties(Filer: TFiler);
begin
 {Define a pseudo property to store the information of the property}
  Filer.DefineProperty('ControlListData', FControlListData, WriteControlListData, true);
end;

Function TResScale.Decompose (var AString:String;Var it:String;Delimiter:String):Boolean;
{Return the next value in a delimited string
 Return the string minus element returned}
Var x:Integer;
begin
   result:=false;
   it:='';
   if Length (Astring) = 0 then Exit;
   x:=pos(Delimiter,Astring); { Get position of delimiter}
   if x = 0  then begin { Return The Last Element}
     it:= Astring;
     Astring:='';
   end
   else begin { return element and remove element from AString}
   It:=Copy(AString,0,(x-1));
   AString:=Copy(Astring,x+1,Length(Astring));
   end;
   result:=true;
end;

constructor TResScale.Create(AOwner: TComponent);
begin
   FormState :=Norm;
   RestoreFormState:=No;
   {Default form position if form is saved with windowstate = wsMaximized}
   {the very first time (only) its position & dimensions are saved}
   DefHeight:= Screen.Height div 2;
   DefWidth:= Screen.Width div 2;
   DefLeft:= Screen.Width div 4;
   DefTop:= Screen.height div 4;
   INHERITED CREATE(Aowner);
   TheOwner:=AOwner;
   GetIniFileName:=DefIniFileName;
   GetScreenHeight:=Screen.height;
   GetScreenWidth:=Screen.Width;
   FStorePositions:=false;
   MyControlList:=TStringList.Create;
End;

procedure TResScale.SetCustomList (Value:TControlsProperty);
begin
       FCustomList:=(Value);
end;

destructor TResScale.Destroy;
begin
  MyControlList.free;
  inherited Destroy;
end;

Procedure TResScale.SetStorePositions(Value:Boolean);
begin
  if value = true then begin
  SaveComponentsPosition;
  if (csDesigning in ComponentState) then begin
   Messagedlg('Component positions for ' + inttostr(GetScreenWidth) + ' and ' +
     inttostr(GetScreenHeight) +' stored in '+ Getinifilename,MTInformation,[MbOk],0);
  end;
  end;
  FStorePositions:=false;
end;

Procedure TResScale.SetRestorePositions(Value:Boolean);
begin
  if Value = true then begin
  LoadComponentsPosition;
  if (csDesigning in ComponentState) then
   Messagedlg('Component positions for ' + inttostr(GetScreenWidth) + ' and ' +
     inttostr(GetScreenHeight) +' restored from '+ Getinifilename,MTInformation,[MbOk],0);
  end;   
  FRestorePositions:=false;
end;

Procedure TResScale.SetScreenWidth (Value:Word);
begin
   GetScreenWidth:=Screen.Width;
end;

Procedure TResScale.SetScreenHeight (Value:Word);
begin
   GetScreenHeight:=Screen.height;
end;

Procedure TResScale.SaveComponentsPosition;
Var
i:integer;
IniFile:TRegInifile;
TheValues,GetSectionName,ControlName:String;
begin
 INIFile := TRegIniFile.Create(GetIniFileName);
 GetSectionName:=Inttostr (Screen.Width)+'*'+inttostr(Screen.height);
 Inifile.EraseSection(GetSectionName);
{Run through control list, for every one in MyControlList save values}
For i:= 0 to TWinControl(TheOwner).controlCount -1  do begin
    ControlName:=TWinControl(TheOwner).Controls[i].Name;
   if MYControlList.IndexOf(ControlName) <> -1 then begin
    with TWinControl(TheOwner).Controls[i] do begin
        TheValues:='';
        TheValues:=Inttostr(Top)+',' + Inttostr(Left)+','
                   +Inttostr(Height)+',' +Inttostr(Width);
    end;
    INIFile.WriteString(GetSectionName,ControlName,TheValues);
   end{if}
end;{for}

if (MyControlList.IndexOf(TheOwner.name) > -1) then begin
   {Its the form save its position & dimensions}
   if Tform(Theowner).windowstate = wsNormal then begin
   TheValues:='';
   TheValues:=Inttostr(TForm(TheOwner).Top)+',' + Inttostr(TForm(TheOwner).Left)+','
   +Inttostr(TForm(TheOwner).Height)+',' +Inttostr(TForm(TheOwner).Width);
   end
   else begin
   {The last normal formstate}
   TheValues:='';
   TheValues:=Inttostr(DefTop)+',' + Inttostr(DefLeft)+','
   +Inttostr(DefHeight)+',' +Inttostr(DefWidth);
   end;
   INIFile.WriteString(GetSectionName,TheOwner.name,TheValues);
end;{if}
   Case Tform(TheOwner).Windowstate of
   wsMaximized:FormState:=Max;
   wsMinimized:Formstate:=Min;
   wsNormal:Formstate:=Norm;
   end;
   INIFile.WriteInteger(GetSectionName,'Formstate',FormState);
end;

Procedure TResScale.LoadComponentsPosition;
Function FindControlIndex(What:TComponentName):Integer;
Var
i:Integer;
Found:Boolean;
begin
  Found:=False;
  i:=0;
  Result:=-1;
  While (Not Found) And (i < TWinControl(Owner).ControlCount) do begin
  If TWinControl(TheOwner).Controls[i].Name = What then begin
  Found := True;
  Result:=i;
  end;
  i:=i+1
  end; {While}
end;
Var
z,i,CompIndex,Saved,TheForm:integer;
IniFile:TRegInifile;
TheValues,GetSectionName,X:String;
SectionValues,SectionKeys:TStringList;
begin
 INIFile := TRegINIFile.Create(GetIniFileName);
 SectionKeys:=TStringlist.Create;
 SectionValues:=TStringlist.Create;
 {Get Current Screen Dimensions and contstruct ini/registry section name from it}
 GetSectionName:=Inttostr (Screen.Width)+'*'+inttostr(Screen.height);
 Inifile.ReadSection (GetSectionName,SectionKeys);
 Inifile.ReadSectionValues (GetSectionName,SectionValues);
 if SectionKeys.count < 1 then begin
 {Load has been called before any save}
 SectionKeys.Free;
 SectionValues.Free;
 Exit;
 end;
 if SectionKeys.Count > 1 then begin
 For i:= 0 to SectionKeys.count -2  do begin
    CompIndex:=FindControlIndex(SectionKeys.Strings[i]); {Check if controlstill exists}
    Saved:=MYControlList.IndexOf(SectionKeys.Strings[i]); {Get index in MyControlList}
    TheForm :=MyControlList.IndexOf(TheOwner.name);{Check if Parent form saved}
   {If Controls name exits and it was part of saved list}
    if ((compIndex > -1) and (Saved > -1)) then begin
    with TWinControl(TheOwner).Controls[CompIndex] do begin
        z:=pos('=',SectionValues.Strings[i]); {get rid of keyname}
        TheValues:=Copy(SectionValues.Strings[i],z+1,Length(SectionValues.Strings[i]));
        {Retore pos & Dimensions}
        Decompose(TheValues,X,',');Top:=strtoInt(x);
        Decompose(TheValues,X,',');Left:=strtoInt(x);
        Decompose(TheValues,X,',');Height:=strtoInt(x);
        Decompose(TheValues,X,',');Width:=strtoInt(x);
    end;
    end{if}
    else begin
      if TheForm > -1 Then begin
      z:=pos('=',SectionValues.Strings[i]); {get rid of keyname}
      TheValues:=Copy(SectionValues.Strings[i],z+1,Length(SectionValues.Strings[i]));
      {Retore pos & Dimensions}
        Decompose(TheValues,X,',');TForm(TheOwner).Top:=strtoInt(x);Deftop:=strtoInt(x);
        Decompose(TheValues,X,',');TForm(TheOwner).Left:=strtoInt(x);DefLeft:=strtoInt(x);
        Decompose(TheValues,X,',');TForm(TheOwner).Height:=strtoInt(x);DefHeight:=strtoInt(x);
        Decompose(TheValues,X,',');TForm(TheOwner).Width:=strtoInt(x);DefWidth:=strtoInt(x);
      end; {if}
    end;{else}
  end; {For}
  end;{if SectionKeys.Count >1}
  i:=SectionKeys.Count-1;
   z:=pos('=',SectionValues.Strings[i]); {get rid of keyname}
  formState:=StrtoInt(Copy(SectionValues.Strings[i],z+1,Length(SectionValues.Strings[i])));
  if ((RestoreFormState = yes) and (Not(csDesigning in ComponentState)))  then
    Case formstate of
    Min: TForm(TheOwner).WindowState := wsminimized;
    Norm: TForm(TheOwner).WindowState := wsNormal;
    Max: TForm(TheOwner).WindowState := wsMaximized;
    end;
 SectionKeys.Free;
 SectionValues.Free;
end;


procedure TResScale.SetIniFileName (Value:String);
begin
     GetIniFileName:=Value;
end;

procedure Register;
begin
  RegisterComponents('Freeware', [TResScale]);
end;

begin
RegisterPropertyEditor(TypeInfo(TControlsProperty), TResScale, 'ControlList',
    TControlsProperty);
end.
